#include <stdio.h>
#include <stdbool.h>
#include <curl/curl.h>
#include <ldap.h>
#include <lber.h>

#include <ssl.h>
#include <nss.h>
#include <nspr.h>
#include <secerr.h>
#include <secmod.h>

static void usage(void)
{
	fprintf(stderr,"usage: ./reproducer <CURL-URI> <LDAP-URI> <ldap-first==1>\n");
}

static bool do_curl(const char *url)
{
	CURL *curl = NULL;
	CURLcode rs;

	curl = curl_easy_init();
	if (!curl) {
		fprintf(stderr, "curl_easy_init() failed\n");
		return false;
	}

	curl_easy_setopt(curl, CURLOPT_URL, url);
	curl_easy_setopt(curl, CURLOPT_NOBODY, 1);
	rs = curl_easy_perform(curl);
	curl_easy_cleanup(curl);

	if (rs != 0) {
		fprintf(stderr, "curl_easy_perform() failed: %s\n", curl_easy_strerror(rs));
		return false;
	}

	fprintf(stderr, "curl done\n");
	return true;
}

static bool do_ldap(const char *url)
{
	LDAP *ldap = NULL;
	int rs;

	rs = ldap_initialize(&ldap, url);
	if (rs != LDAP_SUCCESS) {
		fprintf(stderr, "ldap_initialize() failed: %s\n", ldap_err2string(rs));
		return false;
	}

	int version = 3;
	ldap_set_option(ldap, LDAP_OPT_PROTOCOL_VERSION, &version);

	struct berval password;
	memset(&password, '\0', sizeof(password));

	rs = ldap_sasl_bind_s(ldap, NULL, LDAP_SASL_SIMPLE, &password, NULL, NULL, NULL);
	if (rs != LDAP_SUCCESS) {
		fprintf(stderr, "ldap_sasl_bind_s() failed: %s\n", ldap_err2string(rs));
		return false;
	}

	ldap_unbind_ext(ldap, NULL, NULL);

	fprintf(stderr, "ldap done\n");
	return true;
}

int main(int argc, char *argv[])
{
	if (argc != 4) {
		usage();
		return 1;
	}

	const char *curl_url = argv[1];
	const char *ldap_url = argv[2];
	const bool ldap_first = argv[3][0] == '1';

	//int one = 1;
	//ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, &one);

	if (ldap_first) {
		return (do_ldap(ldap_url) && do_curl(curl_url));
	} else {
		return (do_curl(curl_url) && do_ldap(ldap_url));
	}
}
